using System;
using System.Data;
using System.Windows.Forms;

namespace Kohonen
{
	/// <summary>
	/// 
	/// </summary>
	public class Network
	{
		//Stores network of neurons
		private Neuron [,] net;
		public Neuron [,] Net
		{
			get
			{
				return net;
			}
			set
			{
				net=value;
			}
		}

		//Size of inputs
		private int inSize;
		private int InSize
		{
			get
			{
				return inSize;
			}
			set
			{
				inSize = value;
			}
		}

		//Size of network: x-axis
		private int xSize;
		private int XSize
		{
			get
			{
				return xSize;
			}
			set
			{
				xSize = value;
			}
		}

		//Size of network: y-axis
		private int ySize;
		private int YSize
		{
			get
			{
				return ySize;
			}
			set
			{
				ySize = value;
			}
		}

		//Constructor
		public Network(int inpSize, int x, int y)
		{
			Random rand = new Random();

			//initialise properties
			InSize = inpSize;
			XSize = x;
			YSize = y;

			//initialise neurons with random normalised weights
			net = new Neuron[x,y];
			for (int i=0;i<x;i++)
			{
				for (int j=0;j<y;j++)
				{
					net[i,j] = new Neuron(InSize);
					for (int l=0;l<InSize;l++) 
						net[i,j].Weights[l] = (float)((float)(rand.Next(10000)/(float)30000)+(float)0.001);
					Normalise(ref net[i,j]);
				}
			}
		}

		//Train the network with 1 input, given radius and gain
		public void Train(float [] inputs, int Radius, float Gain)
		{
			int x=XSize;
			int y=YSize;

			//Find winner
			GetWinner(ref x,ref y,inputs);

			//Update its weights
			UpdateWeights(inputs,x,y,Radius,Gain);
		}

		//Find the winner of the network with a given set of input. 
		//Return the score. Used to recognise a known word
		public float GetWinner(ref int x, ref int y, float [] inputs)
		{
			int winnerx = 0;
			int winnery = 0;

			//Iterate over all neurons. Find its output. Minimum is winner.
			float dummy;
			float winnerValue = net[winnerx,winnery].Inputs(inputs);
			for (int i=0;i<XSize;i++)
			{
				for (int j=0;j<YSize;j++)
				{
					dummy = net[i,j].Inputs(inputs);
					if (dummy < winnerValue)
					{
						winnerValue = dummy;
						winnerx = i;
						winnery = j;
					}
				}
			}
			x = winnerx;
			y = winnery;
			return winnerValue;
		}

		//Update weights of all the neurons within the radius from the winner neuron.
		//Gain is the fraction, which neurons are updated with.
		public void UpdateWeights(float [] inputs, int x, int y, int Radius, float Gain)
		{
			float length;
			for (int i=0;i<xSize;i++)
			{
				for (int j=0;j<ySize;j++)
				{
					//length from winner
					length = (float)Math.Sqrt(((i-x)*(i-x)) + ((j-y)*(j-y)));
					length -= Radius;

					//Update the neuron
					if (length <= 0 )
					{
						for (int t=0;t<inSize;t++)
						{
							net[i,j].Weights[t] += Gain * (inputs[t]-net[i,j].Weights[t]);
						}
					}
				}
			}
		}

		//Normalises inputs
		public void Normalise(ref float [] inputs)
		{
			float length = 0;
			for (int i=0;i<inSize;i++)
			{
				length += inputs[i]*inputs[i];
			}
			length = (float)Math.Sqrt(length);
			for (int i=0;i<inSize;i++)
			{
				inputs[i] /= length;
			}
		}

		//C# doesn't allow reference parsing of indexed properties not owned by class.
		public void Normalise(ref Neuron N)
		{
			float length = 0;
			for (int i=0;i<inSize;i++)
			{
				length += N.Weights[i]*N.Weights[i];
			}
			length = (float)Math.Sqrt(length);
			for (int i=0;i<inSize;i++)
			{
				N.Weights[i] /= length;
			}
		}
	}
}
